@@ -1,10 +1,11 @@
 /* Copyright 2015. The Regents of the University of California.
- * Copyright 2015-2016. Martin Uecker.
+ * Copyright 2015-2021. Martin Uecker.
+ + Copyright 2018. Damien Nguyen.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2014-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2014-2021 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  */
 
 #include <stdlib.h>
@@ -15,13 +16,50 @@
 #include <unistd.h>
 #include <errno.h>
 
+#include "misc/io.h"
 #include "misc/misc.h"
+#include "misc/version.h"
+#include "misc/debug.h"
 #include "misc/cppmap.h"
 
+#ifdef USE_CUDA
+#include "num/gpuops.h"
+#endif
+
+#ifdef USE_LOCAL_FFTW
+#include "fftw3_local.h"
+#define MANGLE(name) local_ ## name
+#else
+#include <fftw3.h>
+#define MANGLE(name) name
+#endif
+
 #include "main.h"
 
-struct {
 
+
+extern FILE* bart_output;	// src/misc.c
+
+
+static void bart_exit_cleanup(void)
+{
+	if (NULL != command_line)
+		XFREE(command_line);
+
+	io_memory_cleanup();
+
+#ifdef FFTWTHREADS
+	MANGLE(fftwf_cleanup_threads)();
+#endif
+#ifdef USE_CUDA
+	cuda_memcache_clear();
+#endif
+}
+
+
+
+struct {
+	
 	int (*main_fun)(int argc, char* argv[]);
 	const char* name;
 
@@ -48,7 +86,7 @@ static void usage(void)
 	printf("\n");
 }
 
-int main_bart(int argc, char* argv[])
+int main_bart(int argc, char* argv[argc])
 {
 	char* bn = basename(argv[0]);
 
@@ -57,7 +95,7 @@ int main_bart(int argc, char* argv[])
 		if (1 == argc) {
 
 			usage();
-			exit(1);
+			return 1;
 		}
 
 		const char* tpath[] = {
@@ -86,7 +124,7 @@ int main_bart(int argc, char* argv[])
 				if (ENOENT != errno) {
 
 					perror("Executing bart command failed");
-					exit(1);
+					return 1;
 				}
 
 			} else {
@@ -98,14 +136,47 @@ int main_bart(int argc, char* argv[])
 		return main_bart(argc - 1, argv + 1);
 	}
 
-	for (int i = 0; NULL != dispatch_table[i].name; i++) {
+	unsigned int v[5];
+	version_parse(v, bart_version);
 
+	if (0 != v[4])
+		debug_printf(DP_WARN, "BART version is not reproducible.\n");
+
+	for (int i = 0; NULL != dispatch_table[i].name; i++)
 		if (0 == strcmp(bn, dispatch_table[i].name))
 			return dispatch_table[i].main_fun(argc, argv);
-	}
 
 	fprintf(stderr, "Unknown bart command: \"%s\".\n", bn);
-	exit(1);
+
+	return -1;
 }
 
 
+
+int bart_command(int len, char* buf, int argc, char* argv[])
+{
+	int save = debug_level;
+
+	if (NULL != buf) {
+
+		buf[0] = '\0';
+		bart_output = fmemopen(buf, len, "w");
+	}
+
+	int ret = error_catcher(main_bart, argc, argv);
+
+	bart_exit_cleanup();
+
+	debug_level = save;
+
+	if (NULL != bart_output) {
+
+		fclose(bart_output);	// write final nul
+		bart_output = NULL;
+	}
+
+	return ret;
+}
+
+
+
