@@ -29,7 +29,7 @@ static const char help_str[] = "Compute calibration matrix.";
 
 
 
-int main_calmat(int argc, char* argv[])
+int main_calmat(int argc, char* argv[argc])
 {
 	long calsize[3] = { 24, 24, 24 };
 	long kdims[3] = { 5, 5, 5 };
@@ -103,7 +103,7 @@ int main_calmat(int argc, char* argv[])
 
 	unmap_cfl(N, calmat_dims, out_data);
 
-	exit(0);
+	return 0;
 }
 
 
