@@ -26,7 +26,7 @@ static const char help_str[] = "Kronecker delta.\n";
 
 
 
-int main_delta(int argc, char* argv[])
+int main_delta(int argc, char* argv[argc])
 {
 	mini_cmdline(&argc, argv, 4, usage_str, help_str);
 
@@ -49,7 +49,7 @@ int main_delta(int argc, char* argv[])
 	md_fill_diag(N, dims, flags, x, &(complex float){ 1. }, CFL_SIZE); 
 
 	unmap_cfl(N, dims, x);
-	exit(0);
+	return 0;
 }
 
 
