@@ -12,6 +12,7 @@
 
 #include "num/multind.h"
 #include "num/fft.h"
+#include "num/init.h"
 
 #include "calib/calib.h"
 
@@ -37,7 +38,7 @@ static const char help_str[] =
 
 
 
-int main_ecaltwo(int argc, char* argv[])
+int main_ecaltwo(int argc, char* argv[argc])
 {
 	long maps = 2; // channels;
 	struct ecalib_conf conf = ecalib_defaults;
@@ -54,6 +55,7 @@ int main_ecaltwo(int argc, char* argv[])
 
 	cmdline(&argc, argv, 5, 6, usage_str, help_str, ARRAY_SIZE(opts), opts);
 
+	(conf.usegpu ? num_init_gpu : num_init)();
 
 	long in_dims[DIMS];
 
