@@ -2,7 +2,7 @@
  * Copyright 2016-2017. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
- */ 
+ */
 
 #ifndef __ITALGOS_H
 #define __ITALGOS_H
@@ -14,6 +14,7 @@
 #endif
 
 #include "misc/types.h"
+#include "misc/nested.h"
 
 struct vec_iter_s;
 
@@ -63,11 +64,12 @@ inline void iter_op_p_call(struct iter_op_p_s op, float rho, float* dst, const f
 
 struct iter_monitor_s;
 
-float conjgrad(unsigned int maxiter, float l2lambda, float epsilon, 
+float conjgrad(unsigned int maxiter, float l2lambda, float epsilon,
 	long N,
 	const struct vec_iter_s* vops,
 	struct iter_op_s linop,
-	float* x, const float* b, struct iter_monitor_s* monitor);
+	float* x, const float* b,
+	struct iter_monitor_s* monitor);
 
 
 void landweber(unsigned int maxiter, float epsilon, float alpha,
@@ -76,33 +78,59 @@ void landweber(unsigned int maxiter, float epsilon, float alpha,
 	struct iter_op_s op,
 	struct iter_op_s adj,
 	float* x, const float* b,
+	struct iter_op_s callback,
 	struct iter_monitor_s* monitor);
 
-void landweber_sym(unsigned int maxiter, float epsilon, float alpha,	
+void landweber_sym(unsigned int maxiter, float epsilon, float alpha,
 	long N,
 	const struct vec_iter_s* vops,
 	struct iter_op_s op,
 	float* x, const float* b,
 	struct iter_monitor_s* monitor);
 
-void ist(unsigned int maxiter, float epsilon, float tau, 
-	float continuation, _Bool hogwild,
+
+/**
+ * Store information about iterative algorithm.
+ * Used to flexibly modify behavior, e.g. continuation
+ *
+ * @param rsnew current residual
+ * @param rsnot initial residual
+ * @param iter current iteration
+ * @param maxiter maximum iteration
+ * @param tau tau
+ * @param scale scaling of regularization
+ */
+struct ist_data {
+
+	double rsnew;
+	double rsnot;
+	unsigned int iter;
+	const unsigned int maxiter;
+	float tau;
+	float scale;
+};
+
+typedef void CLOSURE_TYPE(ist_continuation_t)(struct ist_data* itrdata);
+
+
+void ist(unsigned int maxiter, float epsilon, float tau,
 	long N,
 	const struct vec_iter_s* vops,
+	ist_continuation_t ist_continuation,
 	struct iter_op_s op,
 	struct iter_op_p_s thresh,
 	float* x, const float* b,
 	struct iter_monitor_s* monitor);
 
-void fista(unsigned int maxiter, float epsilon, float tau, 
-	float continuation, _Bool hogwild,
+void fista(unsigned int maxiter, float epsilon, float tau,
 	long N,
 	const struct vec_iter_s* vops,
+	ist_continuation_t ist_continuation,
 	struct iter_op_s op,
 	struct iter_op_p_s thresh,
 	float* x, const float* b,
 	struct iter_monitor_s* monitor);
-	
+
 
 void irgnm(unsigned int iter, float alpha, float alpha_min, float redu,
 	long N, long M,
@@ -111,7 +139,18 @@ void irgnm(unsigned int iter, float alpha, float alpha_min, float redu,
 	struct iter_op_s adj,
 	struct iter_op_p_s inv,
 	float* x, const float* x0, const float* y,
-	struct iter_op_s callback);
+	struct iter_op_s callback,
+	struct iter_monitor_s* monitor);
+
+void irgnm2(unsigned int iter, float alpha, float alpha_min, float alpha0, float redu,
+	long N, long M,
+	const struct vec_iter_s* vops,
+	struct iter_op_s op,
+	struct iter_op_s der,
+	struct iter_op_p_s lsqr,
+	float* x, const float* xref, const float* y,
+	struct iter_op_s callback,
+	struct iter_monitor_s* monitor);
 
 void altmin(unsigned int iter, float alpha, float redu,
 	long N,
@@ -133,7 +172,7 @@ double power(unsigned int maxiter,
 	const struct vec_iter_s* vops,
 	struct iter_op_s op,
 	float* u);
-	   
+
 void chambolle_pock(unsigned int maxiter, float epsilon, float tau, float sigma, float theta, float decay,
 	long N, long M,
 	const struct vec_iter_s* vops,
@@ -148,4 +187,3 @@ void chambolle_pock(unsigned int maxiter, float epsilon, float tau, float sigma,
 
 #endif // __ITALGOS_H
 
-
