@@ -70,7 +70,7 @@ static const char help_str[] =
 
 
 
-int main_lrmatrix(int argc, char* argv[])
+int main_lrmatrix(int argc, char* argv[argc])
 {
 	double start_time = timestamp();
 
@@ -174,7 +174,7 @@ int main_lrmatrix(int argc, char* argv[])
 
 	// Initialize operators
 
-	const struct linop_s* sum_op = linop_sum_create(odims);
+	const struct linop_s* sum_op = linop_avg_create(DIMS, odims, LEVEL_FLAG);
 	const struct linop_s* sampling_op = NULL;
 
         if (!decom) {
@@ -187,7 +187,8 @@ int main_lrmatrix(int argc, char* argv[])
 	const struct operator_p_s* sum_prox = prox_lineq_create( sum_op, idata );
 	const struct operator_p_s* lr_prox = lrthresh_create(odims, randshift, mflags, (const long (*)[])blkdims, 1., noise, remove_mean, false);
 
-        assert(use_gpu == false);
+	if (use_gpu)
+		error("GPU support not implemented yet!\n");
 
 	(use_gpu ? num_init_gpu : num_init)();
 
