@@ -4,21 +4,8 @@
 
 #include "misc/mri.h"
 
-struct moba_conf {
-
-	unsigned int iter;
-	float alpha;
-	float alpha_min;
-	float redu;
-	float step;
-	float lower_bound;
-	float tolerance;
-	unsigned int inner_iter;
-	bool noncartesian;
-};
-
-
 
+struct moba_conf;
 extern struct moba_conf moba_defaults;
 
 
