@@ -40,13 +40,13 @@ static const char help_str[] = "Perform non-uniform Fast Fourier Transform.";
 
 
 
-int main_nufft(int argc, char* argv[])
+int main_nufft(int argc, char* argv[argc])
 {
 	bool adjoint = false;
 	bool inverse = false;
 	bool precond = false;
 	bool dft = false;
-	bool gpu = false;
+	bool use_gpu = false;
 
 	struct nufft_conf_s conf = nufft_conf_defaults;
 	struct iter_conjgrad_conf cgconf = iter_conjgrad_defaults;
@@ -68,7 +68,9 @@ int main_nufft(int argc, char* argv[])
 		OPT_UINT('m', &cgconf.maxiter, "", "()"),
 		OPT_SET('P', &conf.periodic, "periodic k-space"),
 		OPT_SET('s', &dft, "DFT"),
-		OPT_SET('g', &gpu, "GPU (only inverse)"),
+		OPT_SET('g', &use_gpu, "GPU (only inverse)"),
+		OPT_CLEAR('1', &conf.decomp, "use/return oversampled grid"),
+		OPTL_SET(0, "lowmem", &conf.lowmem, "Use low-mem mode of the nuFFT"),
 	};
 
 	cmdline(&argc, argv, 3, 3, usage_str, help_str, ARRAY_SIZE(opts), opts);
@@ -77,7 +79,6 @@ int main_nufft(int argc, char* argv[])
 	if (!inverse)
 		conf.toeplitz = false;
 
-
 	long coilim_dims[DIMS] = { 0 };
 	md_copy_dims(3, coilim_dims, coilim_vec);
 
@@ -88,7 +89,7 @@ int main_nufft(int argc, char* argv[])
 	assert(3 == traj_dims[0]);
 
 
-	(gpu ? num_init_gpu : num_init)();
+	(use_gpu ? num_init_gpu : num_init)();
 
 	if (inverse || adjoint) {
 
@@ -102,6 +103,13 @@ int main_nufft(int argc, char* argv[])
 
 			estimate_im_dims(DIMS, FFT_FLAGS, coilim_dims, traj_dims, traj);
 			debug_printf(DP_INFO, "Est. image size: %ld %ld %ld\n", coilim_dims[0], coilim_dims[1], coilim_dims[2]);
+
+			if (!conf.decomp) {
+
+				for (unsigned int i = 0; i < DIMS; i++)
+					if (MD_IS_SET(FFT_FLAGS, i) && (1 < coilim_dims[i]))
+						coilim_dims[i] *= 2;
+			}
 		}
 
 		md_copy_dims(DIMS - 3, coilim_dims + 3, ksp_dims + 3);
@@ -125,7 +133,11 @@ int main_nufft(int argc, char* argv[])
 			if (conf.toeplitz && precond)
 				precond_op = nufft_precond_create(nufft_op);
 
-			lsqr(DIMS, &(struct lsqr_conf){ lambda, gpu }, iter_conjgrad, CAST_UP(&cgconf),
+			struct lsqr_conf lsqr_conf = lsqr_defaults;
+			lsqr_conf.lambda = lambda;
+			lsqr_conf.it_gpu = use_gpu;
+
+			lsqr(DIMS, &lsqr_conf, iter_conjgrad, CAST_UP(&cgconf),
 			     nufft_op, NULL, coilim_dims, img, ksp_dims, ksp, precond_op);
 
 			if (conf.toeplitz && precond)
