@@ -1,10 +1,10 @@
 /* Copyright 2014. The Regents of the University of California.
- * Copyright 2017. Martin Uecker.
+ * Copyright 2017, 2021. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2012, 2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2012-2021 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  */
 
 #include <complex.h>
@@ -30,7 +30,10 @@ struct conv_plan {
 	int N;
 	unsigned int flags;
 
-	struct fft_plan_s* fft_plan;
+	const struct operator_s* fft1;
+	const struct operator_s* ifft1;
+	const struct operator_s* fft2;
+	const struct operator_s* ifft2;
 
 	long* idims;
 	long* odims;
@@ -54,6 +57,9 @@ struct conv_plan {
 struct conv_plan* conv_plan(int N, unsigned int flags, enum conv_type ctype, enum conv_mode cmode, const long odims[N],  
 		const long idims1[N], const long idims2[N], const complex float* src2)
 {
+	assert(   (!((CONV_VALID == ctype) || (CONV_EXTENDED == ctype)))
+	       || (CONV_CAUSAL == cmode));
+
 	PTR_ALLOC(struct conv_plan, plan);
 
 	plan->N = N;
@@ -71,6 +77,8 @@ struct conv_plan* conv_plan(int N, unsigned int flags, enum conv_type ctype, enu
 
 	complex float U = 1.;
 
+	long shift[N];
+
         for (int i = 0; i < N; i++) {
 
 		plan->idims[i] = idims1[i];
@@ -78,22 +86,14 @@ struct conv_plan* conv_plan(int N, unsigned int flags, enum conv_type ctype, enu
 	
 		if (MD_IS_SET(flags, i)) {
 
-			assert(idims2[i] <= idims1[i]);
-
-			if (cmode == CONV_SYMMETRIC) {
-
-				assert((0 == idims1[i] % 2) && (1 == idims2[i] % 2));
-
-				if (1 == (idims1[i] / 2) % 2)
-					U *= -1.i;
-			}
-
 			switch (ctype) {
+
 			case CONV_CYCLIC:
 		
 				assert(odims[i] == idims1[i]);
 				plan->dims1[i] = idims1[i];
 				plan->dims2[i] = odims[i];
+				shift[i] = 0;
 				break;
 
 			case CONV_TRUNCATED:
@@ -101,6 +101,7 @@ struct conv_plan* conv_plan(int N, unsigned int flags, enum conv_type ctype, enu
 				assert(odims[i] == idims1[i]);
 				plan->dims1[i] = idims1[i] + idims2[i] - 1;
 				plan->dims2[i] = odims[i] + idims2[i] - 1;
+				shift[i] = 0;
 				break;
 
 			case CONV_VALID:
@@ -108,6 +109,7 @@ struct conv_plan* conv_plan(int N, unsigned int flags, enum conv_type ctype, enu
 				plan->dims1[i] = odims[i] + idims2[i] - 1;
 				plan->dims2[i] = odims[i] + idims2[i] - 1;
 				assert(idims1[i] == plan->dims1[i]);
+				shift[i] = (1 - idims2[i]);
 				break;
 
 			case CONV_EXTENDED:
@@ -115,6 +117,7 @@ struct conv_plan* conv_plan(int N, unsigned int flags, enum conv_type ctype, enu
 				plan->dims1[i] = idims1[i] + idims2[i] - 1;
 				plan->dims2[i] = idims1[i] + idims2[i] - 1;
 				assert(odims[i] == plan->dims2[i]);
+				shift[i] = 0;
 			}
 
                 	plan->kdims[i] = (1 == idims2[i]) ? 1 : plan->dims1[i];
@@ -137,6 +140,8 @@ struct conv_plan* conv_plan(int N, unsigned int flags, enum conv_type ctype, enu
 			plan->dims1[i] = idims1[i];
 			plan->dims2[i] = odims[i];
 			plan->kdims[i] = idims2[i];
+
+			shift[i] = 0;
 		}
 
 		plan->dims[i] = MAX(plan->dims1[i], plan->dims2[i]);
@@ -150,37 +155,45 @@ struct conv_plan* conv_plan(int N, unsigned int flags, enum conv_type ctype, enu
 	md_calc_strides(N, plan->str2, plan->dims2, CFL_SIZE);
 	md_calc_strides(N, plan->kstr, plan->kdims, CFL_SIZE);
 
-        plan->kernel = md_alloc_sameplace(N, plan->kdims, CFL_SIZE, src2);
-
 	switch (cmode) {
 
 	case CONV_SYMMETRIC:
 
-	        md_resize_center(N, plan->kdims, plan->kernel, idims2, src2, CFL_SIZE);
-       		ifft(N, plan->kdims, flags, plan->kernel, plan->kernel);
-	        fftmod(N, plan->kdims, flags, plan->kernel, plan->kernel);
+		for (int i = 0; i < N; i++)
+			shift[i] -= MD_IS_SET(flags, i) ? ((idims2[i] - 1) / 2) : 0;
+
 		break;
 
 	case CONV_CAUSAL:
 
-	        md_resize(N, plan->kdims, plan->kernel, idims2, src2, CFL_SIZE);
-       		ifft(N, plan->kdims, flags, plan->kernel, plan->kernel);
 		break;
 
 	case CONV_ANTICAUSAL:
 
-	        md_resize(N, plan->kdims, plan->kernel, idims2, src2, CFL_SIZE);
-       		fft(N, plan->kdims, flags, plan->kernel, plan->kernel);
+		for (int i = 0; i < N; i++)
+			shift[i] -= MD_IS_SET(flags, i) ? (idims2[i] - 1) : 0;
+
 		break;
 
 	default:
+
 		assert(0);
 	}
 
-	
+	for (int i = 0; i < N; i++)
+		shift[i] = (plan->kdims[i] + shift[i]) % plan->kdims[i];
+
+	plan->kernel = md_alloc_sameplace(N, plan->kdims, CFL_SIZE, src2);
+
+	md_resize(N, plan->kdims, plan->kernel, idims2, src2, CFL_SIZE);
+	md_circ_shift(N, plan->kdims, shift, plan->kernel, plan->kernel, CFL_SIZE);
+	ifft(N, plan->kdims, flags, plan->kernel, plan->kernel);
         md_zsmul(N, plan->kdims, plan->kernel, plan->kernel, 1. / U);
 
-//	plan->fftplan = fft_plan(N, plan->dims, plan->flags);
+	plan->fft1 = fft_create(N, plan->dims1, plan->flags, NULL, NULL, false);
+	plan->ifft1 = fft_create(N, plan->dims1, plan->flags, NULL, NULL, true);
+	plan->fft2 = fft_create(N, plan->dims2, plan->flags, NULL, NULL, false);
+	plan->ifft2 = fft_create(N, plan->dims2, plan->flags, NULL, NULL, true);
 
 	return PTR_PASS(plan);
 }
@@ -190,9 +203,12 @@ struct conv_plan* conv_plan(int N, unsigned int flags, enum conv_type ctype, enu
 
 void conv_free(struct conv_plan* plan)
 {
-	md_free(plan->kernel);
+	fft_free(plan->fft1);
+	fft_free(plan->ifft1);
+	fft_free(plan->fft2);
+	fft_free(plan->ifft2);
 
-	// fft_free_plan
+	md_free(plan->kernel);
 
 	xfree(plan->dims);
 	xfree(plan->dims1);
@@ -209,81 +225,61 @@ void conv_free(struct conv_plan* plan)
 
 
 
-static void conv_cyclic(struct conv_plan* plan, complex float* dst, const complex float* src1)
+static void conv_cyclic(struct conv_plan* plan, complex float* dst, const complex float* src)
 {
 	// FIXME: optimize tmp away when possible
 	complex float* tmp = md_alloc_sameplace(plan->N, plan->dims1, CFL_SIZE, plan->kernel);
-        ifft(plan->N, plan->dims1, plan->flags, tmp, src1);
+
+	fft_exec(plan->ifft1, tmp, src);
+
 	md_clear(plan->N, plan->dims2, dst, CFL_SIZE);
         md_zfmac2(plan->N, plan->dims, plan->str2, dst, plan->str1, tmp, plan->kstr, plan->kernel);
-  //      md_zmul2(plan->N, plan->dims, plan->str2, dst, plan->str1, tmp, plan->kstr, plan->kernel);
-        fft(plan->N, plan->dims2, plan->flags, dst, dst);
+
+	fft_exec(plan->fft2, dst, dst);
+
 	md_free(tmp);
 }
 
-static void conv_cyclicH(struct conv_plan* plan, complex float* dst, const complex float* src1)
+static void conv_cyclicH(struct conv_plan* plan, complex float* dst, const complex float* src)
 {
-	complex float* tmp = md_alloc_sameplace(plan->N, plan->dims1, CFL_SIZE, plan->kernel);
-        ifft(plan->N, plan->dims2, plan->flags, tmp, src1);
+	complex float* tmp = md_alloc_sameplace(plan->N, plan->dims2, CFL_SIZE, plan->kernel);
+
+	fft_exec(plan->ifft2, tmp, src);
+
 	md_clear(plan->N, plan->dims1, dst, CFL_SIZE);
         md_zfmacc2(plan->N, plan->dims, plan->str1, dst, plan->str2, tmp, plan->kstr, plan->kernel);
-        //md_zmulc2(plan->N, plan->dims1, plan->str1, dst, plan->str2, tmp, plan->kstr, plan->kernel);
-        fft(plan->N, plan->dims1, plan->flags, dst, dst);
+
+	fft_exec(plan->fft1, dst, dst);
+
 	md_free(tmp);
 }
 
 
-
-void conv_exec(struct conv_plan* plan, complex float* dst, const complex float* src1)
+void conv_exec(struct conv_plan* plan, complex float* dst, const complex float* src)
 {
-	bool crop = (CONV_SYMMETRIC == plan->cmode);
-	bool pre = (CONV_TRUNCATED == plan->ctype) || (CONV_EXTENDED == plan->ctype);
-	bool post = (CONV_TRUNCATED == plan->ctype) || (CONV_VALID == plan->ctype);
+	complex float* tmp = md_alloc_sameplace(plan->N, plan->dims, CFL_SIZE, plan->kernel);
 
-	complex float* tmp = NULL;
+	md_resize(plan->N, plan->dims1, tmp, plan->idims, src, CFL_SIZE);
 
-	if (pre || post) {
+	conv_cyclic(plan, tmp, tmp);
 
-		tmp = md_alloc_sameplace(plan->N, plan->dims1, CFL_SIZE, plan->kernel);
-	}
+	md_resize(plan->N, plan->odims, dst, plan->dims2, tmp, CFL_SIZE);
 
-	if (pre)
-		(crop ? md_resize_center : md_resize)(plan->N, plan->dims1, tmp, plan->idims, src1, CFL_SIZE);
-
-	conv_cyclic(plan, post ? tmp : dst, pre ? tmp : src1);
-
-	if (post)
-		(crop ? md_resize_center : md_resize)(plan->N, plan->odims, dst, plan->dims2, tmp, CFL_SIZE);
-
-	if (pre || post)
-		md_free(tmp);
+	md_free(tmp);
 }
 
 
-
-void conv_adjoint(struct conv_plan* plan, complex float* dst, const complex float* src1)
+void conv_adjoint(struct conv_plan* plan, complex float* dst, const complex float* src)
 {
-	bool crop = (CONV_SYMMETRIC == plan->cmode);
-	bool post = (CONV_TRUNCATED == plan->ctype) || (CONV_EXTENDED == plan->ctype);
-	bool pre = (CONV_TRUNCATED == plan->ctype) || (CONV_VALID == plan->ctype);
+	complex float* tmp = md_alloc_sameplace(plan->N, plan->dims, CFL_SIZE, plan->kernel);
 
-	complex float* tmp = NULL;
+	md_resize(plan->N, plan->dims2, tmp, plan->odims, src, CFL_SIZE);
 
-	if (pre || post) {
+	conv_cyclicH(plan, tmp, tmp);
 
-		tmp = md_alloc_sameplace(plan->N, plan->dims1, CFL_SIZE, plan->kernel);
-	}
+	md_resize(plan->N, plan->idims, dst, plan->dims1, tmp, CFL_SIZE);
 
-	if (pre)
-		(crop ? md_resize_center : md_resize)(plan->N, plan->dims2, tmp, plan->odims, src1, CFL_SIZE);
-
-	conv_cyclicH(plan, post ? tmp : dst, pre ? tmp : src1);
-
-	if (post)
-		(crop ? md_resize_center : md_resize)(plan->N, plan->idims, dst, plan->dims1, tmp, CFL_SIZE);
-
-	if (pre || post)
-		md_free(tmp);
+	md_free(tmp);
 }
 
 
