@@ -75,6 +75,6 @@ void md_gaussian_rand(unsigned int D, const long dims[D], complex float* dst)
 #endif
 //#pragma omp parallel for
 	for (long i = 0; i < md_calc_size(D, dims); i++)
-		dst[i] = (float)gaussian_rand();
+		dst[i] = (complex float)gaussian_rand();
 }
 
