@@ -1,13 +1,12 @@
 /* Copyright 2013-2018. The Regents of the University of California.
- * Copyright 2015-2018. Martin Uecker.
+ * Copyright 2015-2019. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2012-2018 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2012-2019 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2014-2016 Frank Ong <frankong@berkeley.edu>
  * 2014-2018 Jon Tamir <jtamir@eecs.berkeley.edu>
- *
  */
 
 #include <assert.h>
@@ -29,7 +28,6 @@
 #include "linops/linop.h"
 #include "linops/fmac.h"
 #include "linops/sampling.h"
-#include "linops/someops.h"
 
 #include "noncart/nufft.h"
 
@@ -55,7 +53,7 @@ static const char help_str[] = "Parallel-imaging compressed-sensing reconstructi
 
 
 
-static const struct linop_s* sense_nc_init(const long max_dims[DIMS], const long map_dims[DIMS], const complex float* maps, const long ksp_dims[DIMS], const long traj_dims[DIMS], const complex float* traj, struct nufft_conf_s conf, const long wgs_dims[DIMS], const complex float* weights, const long basis_dims[DIMS], const complex float* basis, struct operator_s** precond_op, bool sms)
+static const struct linop_s* sense_nc_init(const long max_dims[DIMS], const long map_dims[DIMS], const complex float* maps, const long ksp_dims[DIMS], const long traj_dims[DIMS], const complex float* traj, struct nufft_conf_s conf, const long wgs_dims[DIMS], const complex float* weights, const long basis_dims[DIMS], const complex float* basis, struct operator_s** precond_op)
 {
 	long coilim_dims[DIMS];
 	long img_dims[DIMS];
@@ -72,19 +70,6 @@ static const struct linop_s* sense_nc_init(const long max_dims[DIMS], const long
 
 	const struct linop_s* fft_op = nufft_create2(DIMS, ksp_dims2, coilim_dims, traj_dims, traj, wgs_dims, weights, basis_dims, basis, conf);
 	const struct linop_s* maps_op = maps2_create(coilim_dims, map_dims, img_dims, maps);
-
-	if (sms) {
-
-		/**
-		 * Apply Fourier encoding in image space (after coil
-		 * sensitivity weighting but before NUFFT).
-		 */
-
-		const struct linop_s* fft_slice = linop_fft_create(DIMS, coilim_dims, SLICE_FLAG);
-
-		fft_op = linop_chain_FF(fft_slice, fft_op);
-	}
-
 	const struct linop_s* lop = linop_chain_FF(maps_op, fft_op);
 
 	//precond_op[0] = (struct operator_s*) nufft_precond_create( fft_op );
@@ -94,7 +79,7 @@ static const struct linop_s* sense_nc_init(const long max_dims[DIMS], const long
 }
 
 
-int main_pics(int argc, char* argv[])
+int main_pics(int argc, char* argv[argc])
 {
 	// Initialize default parameters
 
@@ -153,9 +138,9 @@ int main_pics(int argc, char* argv[])
 
 	const struct opt_s opts[] = {
 
-		{ 'l', true, opt_reg, &ropts, "1/-l2\t\ttoggle l1-wavelet or l2 regularization." },
+		{ 'l', NULL, true, opt_reg, &ropts, "1/-l2\t\ttoggle l1-wavelet or l2 regularization." },
 		OPT_FLOAT('r', &ropts.lambda, "lambda", "regularization parameter"),
-		{ 'R', true, opt_reg, &ropts, " <T>:A:B:C\tgeneralized regularization options (-Rh for help)" },
+		{ 'R', NULL, true, opt_reg, &ropts, " <T>:A:B:C\tgeneralized regularization options (-Rh for help)" },
 		OPT_SET('c', &conf.rvc, "real-value constraint"),
 		OPT_FLOAT('s', &step, "step", "iteration stepsize"),
 		OPT_UINT('i', &maxiter, "iter", "max. number of iterations"),
@@ -190,9 +175,10 @@ int main_pics(int argc, char* argv[])
 		OPT_FLOAT('P', &bpsense_eps, "eps", "Basis Pursuit formulation, || y- Ax ||_2 <= eps"),
 		OPT_SELECT('a', enum algo_t, &algo, ALGO_PRIDU, "select Primal Dual"),
 		OPT_SET('M', &sms, "Simultaneous Multi-Slice reconstruction"),
-		OPT_SET('U', &nuconf.lowmem, "Use low-mem mode of the nuFFT"),
+		OPTL_SET('U', "lowmem", &nuconf.lowmem, "Use low-mem mode of the nuFFT"),
 	};
 
+
 	cmdline(&argc, argv, 3, 3, usage_str, help_str, ARRAY_SIZE(opts), opts);
 
 	if (NULL != image_truth_file)
@@ -225,6 +211,11 @@ int main_pics(int argc, char* argv[])
 
         if (sms) {
 
+		if (NULL == traj_file)
+			error("SMS is only supported for non-Cartesian trajectories.\n");
+
+		nuconf.cfft |= SLICE_FLAG;
+
                 debug_printf(DP_INFO, "SMS reconstruction: MB = %ld\n", ksp_dims[SLICE_DIM]);
         }
 
@@ -255,16 +246,17 @@ int main_pics(int argc, char* argv[])
 	md_copy_dims(DIMS, max_dims, ksp_dims);
 	md_copy_dims(5, max_dims, map_dims);
 
-	assert(1 == ksp_dims[COEFF_DIM]);
 	long bmx_dims[DIMS];
 
 	if (NULL != basis_file) {
 
+		assert(1 == ksp_dims[COEFF_DIM]);
+
 		assert(basis_dims[TE_DIM] == ksp_dims[TE_DIM]);
 
 		max_dims[COEFF_DIM] = basis_dims[COEFF_DIM];
 
-		md_copy_dims(DIMS, bmx_dims, max_dims);
+		md_select_dims(DIMS, ~MAPS_FLAG, bmx_dims, max_dims);
 		debug_printf(DP_INFO, "Basis: ");
 		debug_print_dims(DP_INFO, DIMS, bmx_dims);
 
@@ -272,9 +264,6 @@ int main_pics(int argc, char* argv[])
 
 		debug_printf(DP_INFO, "Max:   ");
 		debug_print_dims(DP_INFO, DIMS, max_dims);
-
-//		if (NULL != traj_file)
-//			nuconf.toeplitz = false;
 	}
 
 
@@ -302,7 +291,7 @@ int main_pics(int argc, char* argv[])
 	if (conf.gpu)
 		debug_printf(DP_INFO, "GPU reconstruction\n");
 
-	if (map_dims[MAPS_DIM] > 1) 
+	if (map_dims[MAPS_DIM] > 1)
 		debug_printf(DP_INFO, "%ld maps.\nESPIRiT reconstruction.\n", map_dims[MAPS_DIM]);
 
 	if (conf.bpsense)
@@ -329,6 +318,7 @@ int main_pics(int argc, char* argv[])
 			debug_printf(DP_WARN, "Turning off random shifts\n");
 
 		shift_mode = 2;
+
 		debug_printf(DP_INFO, "Fully overlapping LLR blocks\n");
 	}
 
@@ -357,7 +347,7 @@ int main_pics(int argc, char* argv[])
 
 	if (NULL != traj_file) {
 
-		if (NULL == pat_file && NULL == basis) {
+		if ((NULL == pat_file) && (NULL == basis)) {
 
 			md_free(pattern);
 			pattern = NULL;
@@ -422,7 +412,7 @@ int main_pics(int argc, char* argv[])
 	} else {
 
 		forward_op = sense_nc_init(max_dims, map_dims, maps, ksp_dims, traj_dims, traj, nuconf,
-				pat_dims, pattern, basis_dims, basis, (struct operator_s**)&precond_op, sms);
+				pat_dims, pattern, basis_dims, basis, (struct operator_s**)&precond_op);
 	}
 
 
@@ -445,7 +435,7 @@ int main_pics(int argc, char* argv[])
 		}
 	}
 
-	if (0. == scaling ) {
+	if (0. == scaling) {
 
 		debug_printf(DP_WARN, "Estimated scale is zero. Set to one.");
 		scaling = 1.;
@@ -479,7 +469,9 @@ int main_pics(int argc, char* argv[])
 		if (conf.gpu) {
 
 			complex float* gpu_image_truth = md_gpu_move(DIMS, img_dims, image_truth, CFL_SIZE);
+
 			unmap_cfl(DIMS, img_dims, image_truth);
+
 			image_truth = gpu_image_truth;
 		}
 #endif
@@ -489,7 +481,7 @@ int main_pics(int argc, char* argv[])
 	long img_start_dims[DIMS];
 	complex float* image_start = NULL;
 
-	if (warm_start) { 
+	if (warm_start) {
 
 		debug_printf(DP_DEBUG1, "Warm start: %s\n", image_start_file);
 
@@ -501,7 +493,7 @@ int main_pics(int argc, char* argv[])
 
 		// if rescaling at the end, assume the input has also been rescaled
 		if (scale_im && (scaling != 0.))
-			md_zsmul(DIMS, img_dims, image_start, image_start, 1. /  scaling);
+			md_zsmul(DIMS, img_dims, image_start, image_start, 1. / scaling);
 	}
 
 
@@ -547,12 +539,7 @@ int main_pics(int argc, char* argv[])
 		if (conf.bpsense) {
 
 			const struct linop_s* sample_op = linop_sampling_create(max1_dims, pat1_dims, pattern1);
-			struct linop_s* tmp = linop_chain(forward_op, sample_op);
-
-			linop_free(sample_op);
-			linop_free(forward_op);
-
-			forward_op = tmp;
+			forward_op = linop_chain_FF(forward_op, sample_op);
 		}
 	}
 
@@ -563,7 +550,6 @@ int main_pics(int argc, char* argv[])
 		maxeigen = estimate_maxeigenval(forward_op->normal);
 
 		debug_printf(DP_INFO, "Maximum eigenvalue: %.2e\n", maxeigen);
-
 	}
 
 
@@ -626,16 +612,17 @@ int main_pics(int argc, char* argv[])
 
 	// FIXME: will fail with looped dims
 	struct iter_monitor_s* monitor = NULL;
+
 	if (im_truth)
-		monitor = create_monitor(2*md_calc_size(DIMS, img_dims), (const float*)image_truth, NULL, NULL); 
-	
+		monitor = create_monitor(2*md_calc_size(DIMS, img_dims), (const float*)image_truth, NULL, NULL);
+
 	const struct operator_p_s* po = sense_recon_create(&conf, max1_dims, forward_op,
 				pat1_dims, ((NULL != traj_file) || conf.bpsense) ? NULL : pattern1,
 				it.italgo, it.iconf, image_start1, nr_penalties, thresh_ops,
 				trafos_cond ? trafos : NULL, precond_op, monitor);
 
 	const struct operator_s* op = operator_p_bind(po, 1.);
-//	operator_p_free(po);	// FIXME
+	operator_p_free(po);
 
 	long strsx[2][DIMS];
 	const long* strs[2] = { strsx[0], strsx[1] };
@@ -654,9 +641,15 @@ int main_pics(int argc, char* argv[])
 
 	if (0 != loop_flags) {
 
-		op = operator_copy_wrapper(2, strs, op);
+
+		auto op_tmp = operator_copy_wrapper(2, strs, op);
+		operator_free(op);
+		op = op_tmp;
+
 		// op = operator_loop(DIMS, loop_dims, op);
-		op = operator_loop_parallel(DIMS, loop_dims, op, loop_flags, conf.gpu);
+		op_tmp = operator_loop_parallel(DIMS, loop_dims, op, loop_flags, conf.gpu);
+		operator_free(op);
+		op = op_tmp;
 	}
 
 	operator_apply(op, DIMS, img_dims, image, DIMS, conf.bpsense ? img_dims : ksp_dims, conf.bpsense ? NULL : kspace);
@@ -711,5 +704,3 @@ int main_pics(int argc, char* argv[])
 
 	return 0;
 }
-
-
