@@ -1,10 +1,10 @@
 /* Copyright 2015. The Regents of the University of California.
- * Copyright 2015. Martin Uecker.
+ * Copyright 2015-2020. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2015 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2015-2020 Martin Uecker
  */
 
 #include <complex.h>
@@ -27,27 +27,22 @@ static const char usage_str[] = "flags dim1 ... dimN <input> <output>";
 static const char help_str[] = "Reshape selected dimensions.\n";
 
 
-int main_reshape(int argc, char* argv[])
+int main_reshape(int argc, char* argv[argc])
 {
 	cmdline(&argc, argv, 3, 100, usage_str, help_str, 0, NULL);
 
 	num_init();
 
-	unsigned int flags = atoi(argv[1]);
+	unsigned long flags = atol(argv[1]);
 	unsigned int n = bitcount(flags);
 
 	assert((int)n + 3 == argc - 1);
 
 	long in_dims[DIMS];
-	long in_strs[DIMS];
-
 	long out_dims[DIMS];
-	long out_strs[DIMS];
 
 	complex float* in_data = load_cfl(argv[n + 2], DIMS, in_dims);
 
-	md_calc_strides(DIMS, in_strs, in_dims, CFL_SIZE);
-
 	md_copy_dims(DIMS, out_dims, in_dims);
 	
 	unsigned int j = 0;
@@ -57,21 +52,14 @@ int main_reshape(int argc, char* argv[])
 			out_dims[i] = atoi(argv[j++ + 2]);
 
 	assert(j == n);
-	assert(md_calc_size(DIMS, in_dims) == md_calc_size(DIMS, out_dims));
-
-	md_calc_strides(DIMS, out_strs, out_dims, CFL_SIZE);
-	
-	for (unsigned int i = 0; i < DIMS; i++)
-		if (!(MD_IS_SET(flags, i) || (in_strs[i] == out_strs[i]))) 
-			error("Dimensions are not consistent at index %d.\n", i);
-
 
 	complex float* out_data = create_cfl(argv[n + 3], DIMS, out_dims);
 
-	md_copy(DIMS, in_dims, out_data, in_data, CFL_SIZE);
+	md_reshape(DIMS, flags, out_dims, out_data, in_dims, in_data, CFL_SIZE);
 
 	unmap_cfl(DIMS, in_dims, in_data);
 	unmap_cfl(DIMS, out_dims, out_data);
+
 	return 0;
 }
 
