@@ -47,7 +47,7 @@ static const char usage_str[] = "<lambda> <flags> <input> <output>";
 static const char help_str[] = "Perform total variation denoising along dims <flags>.\n";
 
 	
-int main_rof(int argc, char* argv[])
+int main_rof(int argc, char* argv[argc])
 {
 	mini_cmdline(&argc, argv, 4, usage_str, help_str);
 
