@@ -87,7 +87,7 @@ static void real_from_complex_dims(unsigned int D, long odims[D + 1], const long
 
 
 
-const struct operator_s* sense_recon_create(const struct sense_conf* conf, const long dims[DIMS],
+const struct operator_p_s* sense_recon_create(const struct sense_conf* conf, const long dims[DIMS],
 		  const struct linop_s* sense_op,
 		  const long pat_dims[DIMS], const complex float* pattern,
 		  italgo_fun2_t italgo, iter_conf* iconf,
@@ -95,11 +95,14 @@ const struct operator_s* sense_recon_create(const struct sense_conf* conf, const
 		  unsigned int num_funs,
 		  const struct operator_p_s* thresh_op[num_funs],
 		  const struct linop_s* thresh_funs[num_funs],
-		  const struct operator_s* precond_op)
+		  const struct operator_s* precond_op,
+		  struct iter_monitor_s* monitor)
 {
-	struct lsqr_conf lsqr_conf = { conf->cclambda, conf->gpu };
+	struct lsqr_conf lsqr_conf = lsqr_defaults;
+	lsqr_conf.lambda = conf->cclambda;
+	lsqr_conf.it_gpu = conf->gpu;
 
-	const struct operator_s* op = NULL;
+	const struct operator_p_s* op = NULL;
 
 	assert(DIMS == linop_domain(sense_op)->N);
 
@@ -148,10 +151,10 @@ const struct operator_s* sense_recon_create(const struct sense_conf* conf, const
 
 		if (conf->bpsense)
 			op = lsqr2_create(&lsqr_conf, italgo, iconf, (const float*)init, NULL, NULL,
-					num_funs, thresh_op, thresh_funs, NULL);
+					num_funs, thresh_op, thresh_funs, monitor);
 		else
 			op = lsqr2_create(&lsqr_conf, italgo, iconf, (const float*)init, sense_op, precond_op,
-					num_funs, thresh_op, thresh_funs, NULL);
+					num_funs, thresh_op, thresh_funs, monitor);
 
 		linop_free(sense_op);
 
@@ -174,7 +177,7 @@ const struct operator_s* sense_recon_create(const struct sense_conf* conf, const
 		op = wlsqr2_create(&lsqr_conf, italgo, iconf, (const float*)init,
 						sense_op, weights_op, precond_op,
 						num_funs, thresh_op, thresh_funs,
-						NULL);
+						monitor);
 	}
 
 	return op;
