@@ -79,7 +79,7 @@ static void toimg(bool dicom, bool use_windowing, const char* name, long inum, f
 			} else {
 
 				(*buf)[i][j][0] = (value >> 0) & 0xFF;
-				(*buf)[i][j][2] = (value >> 8) & 0xFF;
+				(*buf)[i][j][1] = (value >> 8) & 0xFF;
 			}
 		}
 	}
@@ -143,7 +143,7 @@ static void toimg_stack(const char* name, bool dicom, bool single_scale, bool us
 }
 
 
-int main_toimg(int argc, char* argv[])
+int main_toimg(int argc, char* argv[argc])
 {
 	float gamma = 1.;
 	float contrast = 0.;
