@@ -1,19 +1,23 @@
-/* Copyright 2015-2016. Martin Uecker
+/* Copyright 2015-2020. Martin Uecker
  * Copyright 2016. The Regents of the University of California.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  * 
  * Authors: 
- * 2015-2-16 Martin Uecker <uecker@med.uni-goettingen.de>
+ * 2015-2020 Martin Uecker <uecker@med.uni-goettingen.de>
  * 2016 Jonathan Tamir <jtamir@eecs.berkeley.edu>
  */
 
 #include <stdbool.h>
 #include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
+#include <assert.h>
 
 #include "misc/misc.h"
 #include "misc/opts.h"
 #include "misc/version.h"
+#include "misc/debug.h"
 
 
 static const char usage_str[] = "[-h]";
@@ -24,19 +28,50 @@ static const char help_str[] =
 	"the number of commits (COMMITS) since this release and\n"
 	"the abbreviated hash of the last commit (SHA). If there\n"
 	"are local changes '-dirty' is added at the end.\n";
+
+
+
 			
 
-int main_version(int argc, char* argv[])
+int main_version(int argc, char* argv[argc])
 {
 	bool verbose = false;
+	const char* version = NULL;
 
 	const struct opt_s opts[] = {
 
+		OPT_STRING('t', &version, "version", "Check minimum version"),
 		OPT_SET('V', &verbose, "Output verbose info"),
 	};
 
 	cmdline(&argc, argv, 0, 0, usage_str, help_str, ARRAY_SIZE(opts), opts);
 
+	if (NULL != version) {
+
+		unsigned int va[5];
+		unsigned int vb[5];
+
+		if (!version_parse(va, bart_version))
+			assert(0);
+
+		if (!version_parse(vb, version))
+			error("Version number not recognized.\n");
+
+		if (0 < vb[4]) { // dirty
+
+			if (0 == strcmp(bart_version, version))
+				return 0;
+
+			debug_printf(DP_WARN, "Comparing to dirty version. Exact match required.\n");
+
+			return 1;
+		}
+
+		// invert here, because unix interprets 0 as succesful exit
+		return !(version_compare(va, vb) >= 0);
+	}
+
+
 	bart_printf("%s\n", bart_version);
 
 	if (verbose) {
@@ -65,6 +100,13 @@ int main_version(int argc, char* argv[])
 #else
 			bart_printf("0\n");
 #endif
+
+		bart_printf("OMP=");
+#ifdef _OPENMP
+			bart_printf("1\n");
+#else
+			bart_printf("0\n");
+#endif
 	}
 
 	return 0;
