@@ -1,6 +1,6 @@
 /* Copyright 2015. The Regents of the University of California.
  * Copyright 2015. Martin Uecker.
- * Copyright 2018. Massachusetts Institute of Technology.
+ * Copyright 2018-2019. Massachusetts Institute of Technology.
  *
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
@@ -8,14 +8,13 @@
  * Authors: 
  * 2015 Berkin Bilgic <berkin@nmr.mgh.harvard.edu>
  * 2015 Martin Uecker <martin.uecker@med.uni-goettingen.de>
- * 2018 Siddharth Iyer <ssi@mit.edu>
+ * 2018-2019 Siddharth Iyer <ssi@mit.edu>
  *
  * B Bilgic, BA Gagoski, SF Cauley, AP Fan, JR Polimeni, PE Grant,
  * LL Wald, and K Setsompop, Wave-CAIPI for highly accelerated 3D
  * imaging. Magn Reson Med (2014) doi: 10.1002/mrm.25347
  */
 
-#include <assert.h>
 #include <stdbool.h>
 #include <complex.h>
 #include <math.h>
@@ -25,6 +24,7 @@
 #include "num/fft.h"
 #include "num/init.h"
 #include "num/iovec.h"
+#include "num/ops_p.h"
 #include "num/ops.h"
 #ifdef USE_CUDA
 #include "num/gpuops.h"
@@ -37,9 +37,7 @@
 #include "linops/linop.h"
 #include "linops/fmac.h"
 #include "linops/someops.h"
-#include "linops/realval.h"
-
-#include "sense/optcom.h"
+#include "linops/decompose_complex.h"
 
 #include "misc/debug.h"
 #include "misc/mri.h"
@@ -49,6 +47,7 @@
 #include "misc/opts.h"
 
 #include "wavelet/wavthresh.h"
+#include "lowrank/lrthresh.h"
 
 static const char usage_str[] = "<maps> <wave> <kspace> <output>";
 static const char help_str[]  = 
@@ -61,12 +60,10 @@ static const char help_str[]  =
 	"                   extended-SENSE model operator\n"
 	"                   dimensions (or # of maps).\n"
 	"Expected dimensions:\n"
-	"  * maps    - (   sx, sy, sz, nc, md)\n"
-	"  * wave    - (   wx, sy, sz,  1,  1)\n"
-	"  * phi     - (    1,  1,  1,  1,  1)\n"
-	"  * output  - (   sx, sy, sz,  1, md)\n"
-	"  * reorder - (    n,  3,  1,  1,  1)\n"
-	"  * table   - (   wx, nc,  n,  1,  1)";
+	"  * maps    - ( sx, sy, sz, nc, md)\n"
+	"  * wave    - ( wx, sy, sz,  1,  1)\n"
+	"  * kspace  - ( wx, sy, sz, nc,  1)\n"
+	"  * output  - ( sx, sy, sz,  1, md)";
 
 /* Helper function to print out operator dimensions. */
 static void print_opdims(const struct linop_s* op) 
@@ -93,7 +90,7 @@ static const struct linop_s* linop_espirit_create(long sx, long sy, long sz, lon
 }
 
 /* Resize operator. */
-static const struct linop_s* linop_reshape_create(long wx, long sx, long sy, long sz, long nc)
+static const struct linop_s* Xlinop_reshape_create(long wx, long sx, long sy, long sz, long nc)
 {
 	long input_dims[] = { [0 ... DIMS - 1] = 1};
 	input_dims[0] = sx;
@@ -157,34 +154,38 @@ static const struct linop_s* linop_samp_create(long wx, long sy, long sz, long n
 
 enum algo_t { CG, IST, FISTA };
 
-int main_wave(int argc, char* argv[])
+int main_wave(int argc, char* argv[argc])
 {
 	double start_time = timestamp();
 
 	float lambda    = 1E-5;
+	int   blksize   = 8;
 	int   maxiter   = 300;
 	float step      = 0.5;
 	float tol       = 1.E-3;
 	bool  wav       = false;
+	bool  llr       = false;
 	bool  fista     = false;
 	bool  hgwld     = false;
 	float cont      = 1;
 	float eval      = -1;
-	int   gpun      = -1;
-	bool  rvc       = false;
+	bool  use_gpu       = false;
+	bool  dcx       = false;
 
 	const struct opt_s opts[] = {
 		OPT_FLOAT( 'r', &lambda,  "lambda", "Soft threshold lambda for wavelet or locally low rank."),
+		OPT_INT(   'b', &blksize, "blkdim", "Block size for locally low rank."),
 		OPT_INT(   'i', &maxiter, "mxiter", "Maximum number of iterations."),
 		OPT_FLOAT( 's', &step,    "stepsz", "Step size for iterative method."),
 		OPT_FLOAT( 'c', &cont,    "cntnu",  "Continuation value for IST/FISTA."),
 		OPT_FLOAT( 't', &tol,     "toler",  "Tolerance convergence condition for iterative method."),
 		OPT_FLOAT( 'e', &eval,    "eigvl",  "Maximum eigenvalue of normal operator, if known."),
-		OPT_INT(   'g', &gpun,    "gpunm",  "GPU device number."),
+		OPT_SET(   'g', &use_gpu,           "use GPU"),
 		OPT_SET(   'f', &fista,             "Reconstruct using FISTA instead of IST."),
 		OPT_SET(   'H', &hgwld,             "Use hogwild in IST/FISTA."),
+		OPT_SET(   'v', &dcx,               "Split result to real and imaginary components."),
 		OPT_SET(   'w', &wav,               "Use wavelet."),
-		OPT_SET(   'v', &rvc,               "Apply real valued constraint on coefficients."),
+		OPT_SET(   'l', &llr,               "Use locally low rank across the real and imaginary components."),
 	};
 
 	cmdline(&argc, argv, 4, 4, usage_str, help_str, ARRAY_SIZE(opts), opts);
@@ -202,10 +203,7 @@ int main_wave(int argc, char* argv[])
 
 	debug_printf(DP_INFO, "Done.\n");
 
-	if (gpun >= 0)
-		num_init_gpu_device(gpun);
-	else
-		num_init();
+	(use_gpu ? num_init_gpu : num_init)();
 
 	int wx = wave_dims[0];
 	int sx = maps_dims[0];
@@ -219,6 +217,7 @@ int main_wave(int argc, char* argv[])
 	recon_dims[1] = sy;
 	recon_dims[2] = sz;
 	recon_dims[4] = md;
+	recon_dims[8] = dcx ? 2 : 1;
 
 	debug_printf(DP_INFO, "FFTMOD maps and kspc... ");
 	fftmod(DIMS, kspc_dims, FFT_FLAGS, kspc, kspc);
@@ -245,7 +244,7 @@ int main_wave(int argc, char* argv[])
 	debug_printf(DP_INFO, "\tE:   %f seconds.\n", t2 - t1);
 
 	t1 = timestamp();
-	const struct linop_s* R = linop_reshape_create(wx, sx, sy, sz, nc);
+	const struct linop_s* R = Xlinop_reshape_create(wx, sx, sy, sz, nc);
 	t2 = timestamp();
 	debug_printf(DP_INFO, "\tR:   %f seconds.\n", t2 - t1);
 
@@ -273,19 +272,19 @@ int main_wave(int argc, char* argv[])
 	struct linop_s* A = linop_chain_FF(linop_chain_FF(linop_chain_FF(linop_chain_FF(linop_chain_FF(
 		E, R), Fx), W), Fyz), M);
 
-	if (rvc == true) {
-		debug_printf(DP_INFO, "\tDomain is restricted to real numbers.\n");
+	if (dcx) {
+		debug_printf(DP_INFO, "\tSplitting result into real and imaginary components.\n");
 		struct linop_s* tmp = A;
-		struct linop_s* rvcop = linop_realval_create(DIMS, linop_domain(A)->dims);
+		struct linop_s* dcxop = linop_decompose_complex_create(DIMS, ITER_DIM, linop_domain(A)->dims);
 
-		A = linop_chain_FF(rvcop, tmp);
+		A = linop_chain_FF(dcxop, tmp);
 	}
 
 	print_opdims(A);
 
 	if (eval < 0)	
 #ifdef USE_CUDA
-		eval = (gpun >= 0) ? estimate_maxeigenval_gpu(A->normal) : estimate_maxeigenval(A->normal);
+		eval = use_gpu ? estimate_maxeigenval_gpu(A->normal) : estimate_maxeigenval(A->normal);
 #else
 		eval = estimate_maxeigenval(A->normal);
 #endif
@@ -298,6 +297,7 @@ int main_wave(int argc, char* argv[])
 	debug_printf(DP_INFO, "Done.\n");
 
 	const struct operator_p_s* T = NULL;
+	long blkdims[MAX_LEV][DIMS];
 	long minsize[] = { [0 ... DIMS - 1] = 1 };
 	minsize[0] = MIN(sx, 16);
 	minsize[1] = MIN(sy, 16);
@@ -305,10 +305,17 @@ int main_wave(int argc, char* argv[])
 	unsigned int WAVFLAG = (sx > 1) * READ_FLAG | (sy > 1) * PHS1_FLAG | (sz > 2) * PHS2_FLAG;
 
 	enum algo_t algo = CG;
-	if (wav == true) {
+	if ((wav) || (llr)) {
 		algo = (fista) ? FISTA : IST;
-		debug_printf(DP_INFO, "Creating wavelet threshold operator... ");
-		T = prox_wavelet_thresh_create(DIMS, recon_dims, WAVFLAG, 0u, minsize, lambda, true);
+		if (wav) {
+			debug_printf(DP_INFO, "Creating wavelet threshold operator... ");
+			T = prox_wavelet_thresh_create(DIMS, recon_dims, WAVFLAG, 0u, minsize, lambda, true);
+			debug_printf(DP_INFO, "Done.\n");
+		} else {
+			debug_printf(DP_INFO, "Creating locally low rank threshold operator across real-imag dimension... ");
+			llr_blkdims(blkdims, ~ITER_FLAG, recon_dims, blksize);
+			T = lrthresh_create(recon_dims, true, ~ITER_FLAG, (const long (*)[])blkdims, lambda, false, false, false);
+		}
 		debug_printf(DP_INFO, "Done.\n");
 	}
 
@@ -388,15 +395,17 @@ int main_wave(int argc, char* argv[])
 
 	debug_printf(DP_INFO, "Reconstruction... ");
 	complex float* recon = create_cfl(argv[4], DIMS, recon_dims);
-	struct lsqr_conf lsqr_conf = { 0., gpun >= 0 };
+	struct lsqr_conf lsqr_conf = lsqr_defaults;
+	lsqr_conf.lambda = 0.;
+	lsqr_conf.it_gpu = use_gpu;
 	double recon_start = timestamp();
-	const struct operator_s* J = lsqr2_create(&lsqr_conf, italgo, iconf, NULL, A, NULL, 1, &T, NULL, NULL);
-	operator_apply(J, DIMS, recon_dims, recon, DIMS, kspc_dims, kspc);
+	const struct operator_p_s* J = lsqr2_create(&lsqr_conf, italgo, iconf, NULL, A, NULL, 1, &T, NULL, NULL);
+	operator_p_apply(J, 1., DIMS, recon_dims, recon, DIMS, kspc_dims, kspc);
 	double recon_end = timestamp();
 	debug_printf(DP_INFO, "Done.\nReconstruction time: %f seconds.\n", recon_end - recon_start);
 
 	debug_printf(DP_INFO, "Cleaning up and saving result... ");
-	operator_free(J);
+	operator_p_free(J);
 	linop_free(A);
 	md_free(mask);
 	unmap_cfl(DIMS, maps_dims, maps);
