@@ -25,7 +25,7 @@ static const char help_str[] = "Create a zero-filled array with {dims} dimension
 
 
 
-int main_zeros(int argc, char* argv[])
+int main_zeros(int argc, char* argv[argc])
 {
 	mini_cmdline(&argc, argv, -3, usage_str, help_str);
 
@@ -47,7 +47,7 @@ int main_zeros(int argc, char* argv[])
 	complex float* x = create_cfl(argv[2 + N], N, dims);
 	md_clear(N, dims, x, sizeof(complex float));
 	unmap_cfl(N, dims, x);
-	exit(0);
+	return 0;
 }
 
 
