@@ -319,11 +319,7 @@ static bool test_ode_sa_bloch(void)
 	for (int i = 0; i < 3; i++)
 		err2 += powf(xp[0][i] - x2[i], 2.);
 
-#ifdef  __clang__
 	if (err2 > 1.E-6)
-#else
-	if (err2 > 1.E-7)
-#endif
 		return false;
 
 	for (int i = 0; i < 3; i++)
